@@ -23,20 +23,18 @@ from tomlkit import document
 from tomlkit.toml_document import TOMLDocument
 
 from ldb import config
-from ldb.config import ConfigType
 from ldb.core import get_ldb_instance
 from ldb.dataset import (
     OpDef,
     apply_queries,
     combine_collections,
-    get_collection_dir_items,
-    get_collection_dir_keys,
     get_collection_from_dataset_identifier,
     iter_collection_dir,
 )
 from ldb.exceptions import DataObjectNotFoundError, LDBException
+from ldb.fs.utils import get_file_hash, unstrip_protocol
 from ldb.index import index
-from ldb.index.utils import expand_indexing_paths
+from ldb.index.utils import FileSystemPath, expand_indexing_paths
 from ldb.path import InstanceDir, WorkspacePath
 from ldb.storage import StorageLocation, get_storage_locations
 from ldb.utils import (
@@ -46,7 +44,6 @@ from ldb.utils import (
     ROOT,
     WORKSPACE_DATASET_PREFIX,
     format_dataset_identifier,
-    get_file_hash,
     get_hash_path,
     parse_data_object_hash_identifier,
     parse_dataset_identifier,
@@ -56,7 +53,6 @@ from ldb.workspace import collection_dir_to_object, load_workspace_dataset
 
 @unique
 class ArgType(Enum):
-    ROOT_DATASET = "root dataset"
     DATASET = "dataset"
     WORKSPACE_DATASET = "workspace_dataset"
     DATA_OBJECT = "data object"
@@ -70,8 +66,6 @@ class AddInput(NamedTuple):
 
 
 def get_arg_type(paths: Sequence[str]) -> ArgType:
-    if any(p == f"{DATASET_PREFIX}{ROOT}" for p in paths):
-        return ArgType.ROOT_DATASET
     if any(p.startswith(DATASET_PREFIX) for p in paths):
         return ArgType.DATASET
     if any(p.startswith(WORKSPACE_DATASET_PREFIX) for p in paths):
@@ -103,30 +97,25 @@ def process_args_for_add(
     return ADD_FUNCTIONS[get_arg_type(paths)](ldb_dir, paths)
 
 
-def root_dataset_for_add(
-    ldb_dir: Path,
-    paths: Sequence[str],  # pylint: disable=unused-argument
-) -> AddInput:
-    data_object_hashes = []
-    annotation_hashes = []
-    for data_object_hash, annotation_hash in get_collection_dir_items(
-        ldb_dir / InstanceDir.DATA_OBJECT_INFO,
-        is_workspace=False,
-    ):
-        data_object_hashes.append(data_object_hash)
-        annotation_hashes.append(annotation_hash or "")
-    return AddInput(data_object_hashes, annotation_hashes, "")
+def parse_dataset_paths(
+    paths: Sequence[str],
+) -> List[Tuple[str, Optional[int]]]:
+    result = []
+    for path in paths:
+        try:
+            name, version = parse_dataset_identifier(path)
+        except ValueError as exc:
+            raise ValueError(
+                "All paths must be the same type. "
+                f"Found dataset identifier, but unable to "
+                f"parse all arguments as datasets: {path}",
+            ) from exc
+        result.append((name, version))
+    return result
 
 
 def dataset_for_add(ldb_dir: Path, paths: Sequence[str]) -> AddInput:
-    try:
-        dataset_identifiers = [parse_dataset_identifier(p) for p in paths]
-    except LDBException as exc:
-        raise LDBException(
-            "All paths must be the same type. "
-            "Found path starting with 'ds', but unable "
-            "parse all paths as a dataset identifier",
-        ) from exc
+    dataset_identifiers = parse_dataset_paths(paths)
     collections = [
         get_collection_from_dataset_identifier(
             ldb_dir,
@@ -145,6 +134,8 @@ def dataset_for_add(ldb_dir: Path, paths: Sequence[str]) -> AddInput:
 
 def workspace_dataset_for_add(ldb_dir: Path, paths: Sequence[str]) -> AddInput:
     paths = [re.sub(r"^ws:", "", p) for p in paths]
+    for path in paths:
+        load_workspace_dataset(Path(path))
     collections = [
         collection_dir_to_object(
             Path(path) / WorkspacePath.COLLECTION,
@@ -185,19 +176,19 @@ def path_for_add(ldb_dir: Path, paths: Sequence[str]) -> AddInput:
         data_object_hashes_from_path(paths, get_storage_locations(ldb_dir)),
     )
     try:
-        annotation_hashes = get_current_annotation_hashes(
+        annotation_hashes = get_current_annotation_hashes_from_file_hashes(
             ldb_dir,
             data_object_hash_iter,
         )
     except DataObjectNotFoundError as exc:
-        cfg: TOMLDocument = (
-            config.load_first([ConfigType.INSTANCE]) or document()
-        )
+        cfg: TOMLDocument = config.load_first() or document()
         auto_index: bool = cfg.get("core", {}).get("auto_index", False)
         if not auto_index:
             raise DataObjectNotFoundError(
-                f"{exc!s}\n"
-                f"Will not index new data: auto_index = {auto_index}",
+                f"{exc!s}\n\n"
+                "Encountered data object not previously indexed by LDB. "
+                "Please index first or enable auto-indexing by setting:\n\n"
+                "\tcore.auto_index = true\n",
             ) from exc
 
         indexing_result = index(
@@ -214,7 +205,7 @@ def path_for_add(ldb_dir: Path, paths: Sequence[str]) -> AddInput:
         )
         message = indexing_result.summary()
     else:
-        data_object_hashes = list(data_object_hash_iter2)
+        data_object_hashes = [h.value for h in data_object_hash_iter2]
         message = ""
 
     return AddInput(
@@ -225,7 +216,6 @@ def path_for_add(ldb_dir: Path, paths: Sequence[str]) -> AddInput:
 
 
 ADD_FUNCTIONS: Dict[ArgType, Callable[[Path, Sequence[str]], AddInput]] = {
-    ArgType.ROOT_DATASET: root_dataset_for_add,
     ArgType.DATASET: dataset_for_add,
     ArgType.WORKSPACE_DATASET: workspace_dataset_for_add,
     ArgType.DATA_OBJECT: data_object_for_add,
@@ -324,26 +314,8 @@ def process_args_for_delete(
     return DELETE_FUNCTIONS[arg_type](ldb_dir, paths)
 
 
-def root_dataset_for_delete(
-    ldb_dir: Path,
-    paths: Sequence[str],  # pylint: disable=unused-argument
-) -> List[str]:
-    return sorted(
-        get_collection_dir_keys(
-            ldb_dir / InstanceDir.DATA_OBJECT_INFO,
-        ),
-    )
-
-
 def dataset_for_delete(ldb_dir: Path, paths: Sequence[str]) -> List[str]:
-    try:
-        dataset_identifiers = [parse_dataset_identifier(p) for p in paths]
-    except LDBException as exc:
-        raise LDBException(
-            "All paths must be the same type. "
-            "Found path starting with 'ds', but unable "
-            "parse all paths as a dataset identifier",
-        ) from exc
+    dataset_identifiers = parse_dataset_paths(paths)
     collections = [
         get_collection_from_dataset_identifier(
             ldb_dir,
@@ -383,9 +355,13 @@ def path_for_delete(
     ldb_dir: Path,
     paths: Sequence[str],
 ) -> List[str]:
-    return list(
-        data_object_hashes_from_path(paths, get_storage_locations(ldb_dir)),
-    )
+    return [
+        f.value
+        for f in data_object_hashes_from_path(
+            paths,
+            get_storage_locations(ldb_dir),
+        )
+    ]
 
 
 def get_data_object_storage_files(
@@ -402,16 +378,20 @@ def get_data_object_storage_files(
                 yield fs, path
 
 
+class FileHash(NamedTuple):
+    fs_path: FileSystemPath
+    value: str
+
+
 def data_object_hashes_from_path(
     paths: Iterable[str],
     storage_locations: Iterable[StorageLocation],
-) -> Iterator[str]:
+) -> Iterator[FileHash]:
     for fs, path in get_data_object_storage_files(paths, storage_locations):
-        yield get_file_hash(fs, path)
+        yield FileHash(FileSystemPath(fs, path), get_file_hash(fs, path))
 
 
 DELETE_FUNCTIONS: Dict[ArgType, Callable[[Path, Sequence[str]], List[str]]] = {
-    ArgType.ROOT_DATASET: root_dataset_for_delete,
     ArgType.DATASET: dataset_for_delete,
     ArgType.WORKSPACE_DATASET: workspace_dataset_for_delete,
     ArgType.DATA_OBJECT: data_object_for_delete,
@@ -519,6 +499,26 @@ def get_current_annotation_hashes(
     ]
 
 
+def get_current_annotation_hashes_from_file_hashes(
+    ldb_dir: Path,
+    data_object_hashes: Iterable[FileHash],
+) -> List[str]:
+    result = []
+    for file_hash in data_object_hashes:
+        try:
+            annot_hash = get_current_annotation_hash(ldb_dir, file_hash.value)
+        except DataObjectNotFoundError:
+            fs, path = file_hash.fs_path
+            path = unstrip_protocol(fs, path)
+            raise DataObjectNotFoundError(  # pylint: disable=raise-missing-from # noqa: E501
+                "Data object not found: "
+                f"{DATA_OBJ_ID_PREFIX}{file_hash.value} "
+                f"(path={path})",
+            )
+        result.append(annot_hash)
+    return result
+
+
 def process_args_for_ls(
     ldb_dir: Path,
     paths: Sequence[str],
@@ -564,7 +564,6 @@ def path_for_ls(ldb_dir: Path, paths: Sequence[str]) -> AddInput:
 
 
 LS_FUNCTIONS: Dict[ArgType, Callable[[Path, Sequence[str]], AddInput]] = {
-    ArgType.ROOT_DATASET: root_dataset_for_add,
     ArgType.DATASET: dataset_for_add,
     ArgType.WORKSPACE_DATASET: workspace_dataset_for_add,
     ArgType.DATA_OBJECT: data_object_for_add,
