@@ -11,6 +11,14 @@ from ldb.op_type import OpType
 SIMPLE_NAME_PATTERN = r"^[^\s,]+$"
 
 
+def json_bool(text: str) -> bool:
+    if text == "true":
+        return True
+    if text == "false":
+        return False
+    raise ValueError(f"Not a JSON bool: {text}")
+
+
 def simple_name_list(value: str) -> List[str]:
     result = []
     for item in value.split(","):
@@ -212,20 +220,24 @@ def add_target_dir_argument(parser: ArgumentParser) -> None:
     ).complete = shtab.DIR
 
 
-def add_instantiate_arguments(parser: ArgumentParser) -> None:
+def add_instantiate_arguments(
+    parser: ArgumentParser,
+    include_force: bool = True,
+) -> None:
     add_data_obj_params(parser, dest="query_args")
     add_data_format_arguments(
         parser,
         default=Format.BARE,
         formats=INSTANTIATE_FORMATS,
     )
-    parser.add_argument(
-        "-f",
-        "--force",
-        action="store_true",
-        default=False,
-        help="Remove existing workspace contents",
-    )
+    if include_force:
+        parser.add_argument(
+            "-f",
+            "--force",
+            action="store_true",
+            default=False,
+            help="Remove existing workspace contents",
+        )
     add_target_dir_argument(parser)
     parser.add_argument(
         "--apply",
