@@ -38,6 +38,7 @@ from ldb.query.search import (
 )
 from ldb.typing import JSONDecoded, JSONObject
 from ldb.utils import (
+    ROOT,
     format_dataset_identifier,
     format_datetime,
     get_hash_path,
@@ -79,9 +80,10 @@ class DatasetVersion:
     version: int
     parent: str
     collection: str
+    transform_mapping_id: str
     tags: List[str]
     commit_info: CommitInfo
-    transform_mapping_id: str = ""
+    auto_pull: bool = False
 
     @classmethod
     def parse(cls, attr_dict: Dict[str, Any]) -> "DatasetVersion":
@@ -91,6 +93,21 @@ class DatasetVersion:
             **attr_dict,
         )
 
+    @classmethod
+    def from_id(
+        cls,
+        ldb_dir: Path,
+        dataset_version_id: str,
+    ) -> "DatasetVersion":
+        return cls.parse(
+            load_data_file(
+                get_hash_path(
+                    ldb_dir / InstanceDir.DATASET_VERSIONS,
+                    dataset_version_id,
+                ),
+            ),
+        )
+
     def format(self) -> Dict[str, Any]:
         attr_dict = {f.name: getattr(self, f.name) for f in fields(self)}
         return dict(
@@ -127,6 +144,17 @@ def iter_collection_dir(collection_dir: Union[str, Path]) -> Iterator[str]:
     return iglob(os.path.join(collection_dir, "*", "*"))
 
 
+def get_root_collection(
+    ldb_dir: Path,
+) -> Dict[str, Optional[str]]:
+    return dict(
+        get_collection_dir_items(
+            ldb_dir / InstanceDir.DATA_OBJECT_INFO,
+            is_workspace=False,
+        ),
+    )
+
+
 def get_collection(
     ldb_dir: Path,
     dataset_version_hash: str,
@@ -152,6 +180,8 @@ def get_collection_from_dataset_identifier(
     dataset_name: str,
     dataset_version: Optional[int] = None,
 ) -> Dict[str, Optional[str]]:
+    if dataset_name == ROOT:
+        return get_root_collection(ldb_dir)
     dataset = get_dataset(ldb_dir, dataset_name)
     dataset_version_hash = get_dataset_version_hash(dataset, dataset_version)
     return get_collection(ldb_dir, dataset_version_hash)
