@@ -18,12 +18,18 @@ from typing import (
 from fsspec.spec import AbstractFileSystem
 from funcy.objects import cached_property
 
-from ldb.dataset import get_collection_dir_keys
+from ldb.dataset import (
+    get_annotation,
+    get_collection_dir_keys,
+    get_root_collection_annotation_hash,
+)
 from ldb.exceptions import IndexingException, LDBException
+from ldb.fs.utils import get_file_hash, get_modified_time
 from ldb.index.utils import (
     DEFAULT_CONFIG,
     INDEXED_EPHEMERAL_CONFIG,
     AnnotationMeta,
+    AnnotMergeStrategy,
     DataObjectMeta,
     DataToWrite,
     FileSystemPath,
@@ -47,12 +53,10 @@ from ldb.typing import JSONDecoded, JSONObject
 from ldb.utils import (
     current_time,
     format_datetime,
-    get_file_hash,
     get_hash_path,
     hash_data,
     json_dumps,
     load_data_file,
-    timestamp_to_datetime,
     write_data_file,
 )
 
@@ -64,6 +68,7 @@ class IndexedObjectResult(NamedTuple):
     found_annotation: bool
     new_data_object: bool
     new_annotation: bool
+    new_data_object_path: bool
     data_object_hash: str
 
 
@@ -73,6 +78,7 @@ class IndexingResult:
     num_found_annotations: int = 0
     num_new_data_objects: int = 0
     num_new_annotations: int = 0
+    num_new_data_object_paths: int = 0
     data_object_hashes: List[str] = field(default_factory=list)
 
     def append(self, item: IndexedObjectResult) -> None:
@@ -80,6 +86,7 @@ class IndexingResult:
         self.num_found_annotations += item.found_annotation
         self.num_new_data_objects += item.new_data_object
         self.num_new_annotations += item.new_annotation
+        self.num_new_data_object_paths += item.new_data_object_path
         self.data_object_hashes.append(item.data_object_hash)
 
     def summary(self, finished: bool = True) -> str:
@@ -89,10 +96,11 @@ class IndexingResult:
             heading = "Unable to finish indexing. Partial indexing results"
         return (
             f"\n{heading}:\n"
-            f"  Found data objects: {self.num_found_data_objects:9d}\n"
-            f"  Found annotations:  {self.num_found_annotations:9d}\n"
-            f"  New data objects:   {self.num_new_data_objects:9d}\n"
-            f"  New annotations:    {self.num_new_annotations:9d}"
+            f"  Found data objects:    {self.num_found_data_objects:9d}\n"
+            f"  Found annotations:     {self.num_found_annotations:9d}\n"
+            f"  New data objects:      {self.num_new_data_objects:9d}\n"
+            f"  New annotations:       {self.num_new_annotations:9d}\n"
+            f"  New data object paths: {self.num_new_data_object_paths:9d}"
         )
 
 
@@ -138,10 +146,12 @@ class Indexer(ABC):
         ldb_dir: Path,
         preprocessor: Preprocessor,
         tags: Collection[str] = (),
+        annot_merge_strategy: AnnotMergeStrategy = AnnotMergeStrategy.REPLACE,
     ) -> None:
         self.ldb_dir = ldb_dir
         self.preprocessor = preprocessor
         self.tags = tags
+        self.annot_merge_strategy = annot_merge_strategy
         self.result = IndexingResult()
         self.hashes: Dict[AbstractFileSystem, Dict[str, str]] = {}
 
@@ -167,8 +177,9 @@ class PairIndexer(Indexer):
         read_any_cloud_location: bool,
         strict_format: bool,
         tags: Collection[str] = (),
+        annot_merge_strategy: AnnotMergeStrategy = AnnotMergeStrategy.REPLACE,
     ) -> None:
-        super().__init__(ldb_dir, preprocessor, tags)
+        super().__init__(ldb_dir, preprocessor, tags, annot_merge_strategy)
         self.read_any_cloud_location = read_any_cloud_location
         self.strict_format = strict_format
         self.old_to_new_files: Dict[
@@ -236,7 +247,13 @@ class PairIndexer(Indexer):
             )
             if read_add_location is None:
                 raise LDBException(
-                    "No read-add storage configured. See 'ldb add-storage -h'",
+                    "Local paths outside of configured storage locations "
+                    "found, but no read-add storage configured.\n"
+                    "Either add storage locations containing all indexing "
+                    "paths or add a read-add storage location to handle "
+                    "ephemeral data paths.\n\n"
+                    "For more information, run:\n\n"
+                    "\tldb add-storage --help\n",
                 )
             (
                 self.old_to_new_files,
@@ -248,6 +265,20 @@ class PairIndexer(Indexer):
                 self.hashes,
                 strict_format=self.strict_format,
             )
+            num_ephemeral_data_objects = sum(
+                map(len, self.old_to_new_files.values()),
+            )
+            num_ephemeral_annotations = sum(
+                map(len, self.old_to_new_annot_files.values()),
+            )
+            path = read_add_location.path
+            if read_add_location.protocol != "file":
+                path = f"{read_add_location.protocol}://{path}"
+            print(
+                f"\nCopied ephemeral files to storage: {path}\n"
+                f"  Data objects:       {num_ephemeral_data_objects:9d}\n"
+                f"  Annotations:        {num_ephemeral_annotations:9d}",
+            )
 
             for _, fs_paths in self.old_to_new_annot_files.items():
                 for _, new in fs_paths.items():
@@ -292,6 +323,7 @@ class PairIndexer(Indexer):
                             config.save_data_object_path_info,
                             annotation_path,
                             self.tags,
+                            self.annot_merge_strategy,
                         )
                         self.result.append(obj_result)
 
@@ -302,11 +334,13 @@ class PairIndexer(Indexer):
         save_data_object_path_info: bool,
         annotation_path: str,
         tags: Collection[str],
+        annot_merge_strategy: AnnotMergeStrategy,
     ) -> IndexedObjectResult:
         return PairIndexingItem(
             self.ldb_dir,
             current_time(),
             tags,
+            annot_merge_strategy,
             FileSystemPath(fs, data_object_path),
             save_data_object_path_info,
             self.hashes,
@@ -323,6 +357,8 @@ class IndexingItem(ABC):
         default_factory=list,
     )
     tags: Collection[str]
+    annot_merge_strategy: AnnotMergeStrategy
+    found_new_data_object_path: bool = field(init=False, default=False)
 
     @cached_property
     def current_timestamp(self) -> str:
@@ -382,9 +418,36 @@ class IndexingItem(ABC):
         raise NotImplementedError
 
     @cached_property
-    def annotation_content(self) -> JSONDecoded:
+    def raw_annotation_content(self) -> JSONDecoded:
         raise NotImplementedError
 
+    @cached_property
+    def annotation_content(self) -> JSONDecoded:
+        if self.annot_merge_strategy == AnnotMergeStrategy.REPLACE:
+            return self.raw_annotation_content
+        if self.annot_merge_strategy == AnnotMergeStrategy.MERGE:
+            return self.get_merged_annotation_content()
+        raise ValueError(
+            "Invalid annotation merge strategy: "
+            f"{self.annot_merge_strategy}",
+        )
+
+    def get_merged_annotation_content(self) -> JSONDecoded:
+        if not isinstance(self.raw_annotation_content, Mapping):
+            return self.raw_annotation_content
+        last_annot_hash = get_root_collection_annotation_hash(
+            get_hash_path(
+                self.ldb_dir / InstanceDir.DATA_OBJECT_INFO,
+                self.data_object_hash,
+            ),
+        )
+        if last_annot_hash is None:
+            return self.raw_annotation_content
+        last_annot = get_annotation(self.ldb_dir, last_annot_hash)
+        if not isinstance(last_annot, Mapping):
+            return self.raw_annotation_content
+        return {**last_annot, **self.raw_annotation_content}
+
     @cached_property
     def annotation_ldb_content_bytes(self) -> bytes:
         return json_dumps(self.annotation_ldb_content).encode()
@@ -420,6 +483,7 @@ class IndexingItem(ABC):
             found_annotation=found_annotation,
             new_data_object=new_data_object,
             new_annotation=new_annotation,
+            new_data_object_path=self.found_new_data_object_path,
             data_object_hash=self.data_object_hash,
         )
 
@@ -477,14 +541,6 @@ class AnnotationFileIndexingItem(IndexingItem):
     def annotation_meta(self) -> AnnotationMeta:
         if self.annotation_fsp is None:
             raise IndexingException("Missing annotation_fsp")
-        fs_info = self.annotation_fsp.fs.info(self.annotation_fsp.path)
-        curr_mtime_float = fs_info.get("created")
-        if curr_mtime_float is not None:
-            curr_mtime: Optional[datetime] = timestamp_to_datetime(
-                curr_mtime_float,
-            )
-        else:
-            curr_mtime = None
         prev_annotation = (
             load_data_file(self.annotation_meta_file_path)
             if self.annotation_meta_file_path.exists()
@@ -494,11 +550,14 @@ class AnnotationFileIndexingItem(IndexingItem):
             prev_annotation,
             self.current_timestamp,
             self.annotation_version,
-            curr_mtime,
+            get_modified_time(
+                self.annotation_fsp.fs,
+                self.annotation_fsp.path,
+            ),
         )
 
     @cached_property
-    def annotation_content(self) -> JSONDecoded:
+    def raw_annotation_content(self) -> JSONDecoded:
         if self.annotation_fsp is None:
             raise IndexingException("Missing annotation_fsp")
         return get_annotation_content(
@@ -537,11 +596,17 @@ class DataObjectFileIndexingItem(IndexingItem):
                 | set(self.tags),
             )
         else:
-            meta_contents = construct_data_object_meta(
-                *self.data_object,
+            old_meta = (
                 load_data_file(self.data_object_meta_file_path)
                 if self.data_object_meta_file_path.exists()
-                else {},
+                else {}
+            )
+            (
+                self.found_new_data_object_path,
+                meta_contents,
+            ) = construct_data_object_meta(
+                *self.data_object,
+                old_meta,
                 self.current_timestamp,
                 tags=self.tags,
             )
