@@ -1,13 +1,14 @@
 import os
 import shutil
 from pathlib import Path
-from typing import Dict, List, Optional, Tuple
+from typing import Dict, List, Optional, Sequence, Tuple
 
 from ldb.config import get_ldb_dir
 from ldb.core import init_quickstart
 from ldb.dataset import Dataset, DatasetVersion
 from ldb.exceptions import LDBException
 from ldb.path import InstanceDir, WorkspacePath
+from ldb.pull import get_collection_with_updated_annotations
 from ldb.utils import (
     DATASET_PREFIX,
     ROOT,
@@ -107,6 +108,7 @@ def stage_with_instance(  # pylint: disable=too-many-statements
         staged_time=current_time(),
         parent="",
         tags=[],
+        auto_pull=False,
     )
     try:
         dataset_obj = Dataset.parse(
@@ -120,6 +122,7 @@ def stage_with_instance(  # pylint: disable=too-many-statements
                 f"To stage a new dataset, use {ds_ident}",
             ) from exc
         collection_obj = None
+        transform_obj = None
         message = (
             f"Staged new dataset {dataset_identifier} "
             f"at {os.fspath(workspace_path)!r}"
@@ -150,12 +153,26 @@ def stage_with_instance(  # pylint: disable=too-many-statements
         )
         workspace_ds_obj.parent = dataset_version_hash
         workspace_ds_obj.tags = dataset_version_obj.tags.copy()
+        workspace_ds_obj.auto_pull = dataset_version_obj.auto_pull
         collection_obj = load_data_file(
             get_hash_path(
                 ldb_dir / InstanceDir.COLLECTIONS,
                 dataset_version_obj.collection,
             ),
         )
+        if workspace_ds_obj.auto_pull:
+            collection_obj = dict(
+                get_collection_with_updated_annotations(
+                    ldb_dir,
+                    collection_obj.keys(),
+                )[0],
+            )
+        transform_obj = load_data_file(
+            get_hash_path(
+                ldb_dir / InstanceDir.TRANSFORM_MAPPINGS,
+                dataset_version_obj.transform_mapping_id,
+            ),
+        )
         curr_dataset_ident = format_dataset_identifier(
             ds_name,
             ds_version_num,
@@ -163,7 +180,12 @@ def stage_with_instance(  # pylint: disable=too-many-statements
         message = (
             f"Staged {curr_dataset_ident} at {os.fspath(workspace_path)!r}"
         )
-    stage_workspace(workspace_path, workspace_ds_obj, collection_obj)
+    stage_workspace(
+        workspace_path,
+        workspace_ds_obj,
+        collection_obj,
+        transform_obj,
+    )
     print(message)
 
 
@@ -171,6 +193,7 @@ def stage_workspace(
     workspace_path: Path,
     workspace_ds_obj: WorkspaceDataset,
     collection_obj: Optional[Dict[str, Optional[str]]] = None,
+    transform_obj: Optional[Dict[str, Sequence[str]]] = None,
 ) -> None:
     collection_path = workspace_path / WorkspacePath.COLLECTION
     workspace_ds_bytes = json_dumps(workspace_ds_obj.format()).encode()
@@ -186,6 +209,22 @@ def stage_workspace(
             shutil.rmtree(collection_path)
         collection_path.mkdir(parents=True)
 
+    transform_path = workspace_path / WorkspacePath.TRANSFORM_MAPPING
+    if transform_obj:
+        transform_path_data = [
+            (
+                get_hash_path(transform_path, data_object_hash),
+                json_dumps(transform_ids),
+            )
+            for data_object_hash, transform_ids in transform_obj.items()
+        ]
+        transform_path.mkdir(parents=True, exist_ok=True)
+        write_workspace_collection(transform_path, transform_path_data)
+    else:
+        if transform_path.exists():
+            shutil.rmtree(transform_path)
+        transform_path.mkdir(parents=True)
+
     write_data_file(
         workspace_path / WorkspacePath.DATASET,
         workspace_ds_bytes,
